/*============================================================
 * Name  : httask_mgr.c
 *
 * Desc  : ^XN}l[W[
 *
 * prefix: HTTask_
 *
 * $Id: httask_mgr.c,v 1.1 1998/04/27 07:07:17 horita Exp $
 *
 * $Log: httask_mgr.c,v $
 * Revision 1.1  1998/04/27 07:07:17  horita
 * Initial revision
 *
 *============================================================*/

#if DOCUMENT
#endif

/*============================================================
  Include
  ============================================================*/
#include <stdio.h>
#include <string.h>
#include "htmem_mgr.h"
#include "httask_mgr.h"

/*============================================================
  Macro
  ============================================================*/
#define ASG_BIT(NAME, NUM) NAME = (1 << NUM)

/*============================================================
  Define
  ============================================================*/
enum {
	TSKMODE_NULL,									// Ȃ
	TSKMODE_CTRL,									// 䃋[v
	TSKMODE_DRAW,									// `惋[v
};

/*============================================================
  Structure
  ============================================================*/

/*============================================================
  Globals
  ============================================================*/
int task_no;										/* ^XN */
struct taskheader *Curtask;							/* ݎs̃^XN */

/*============================================================
  Static
  ============================================================*/
static struct taskheader *RootTask;					/* [g̃^XN */
static void* init_param = NULL;						/* p[^ */
static int task_mode = 0;							// ^XN̏[h

/*============================================================
  Local Prototypes
  ============================================================*/
static void HTTask_Leave(struct taskheader* p);

/*============================================================
 Name:
 Desc:
 Kind: Global
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/04/27 (Monday)
 Misc:
 ------------------------------------------------------------
 in  :
 out :
 ============================================================*/
void* HTTask_CreateChild(const void *leader, int size, const HTTaskFunc* FuncTbl)
{
    register struct taskheader *p;
    register struct taskheader *s;

    if ((p = HTMem_Alloc(size + TASK_HEADER)) != NULL) {
		register struct taskheader *_leader = (struct taskheader *) ((char *) leader - TASK_HEADER);
		memset(p, 0, size + TASK_HEADER);
		if (FuncTbl->init != NULL) {				/* s֐̎s */
			if (init_param == NULL) {
				register FuncP func = (FuncP) FuncTbl->init;
				func(p->body);
			} else {
				void (*func)(void *) = FuncTbl->init;
				func(p->body, init_param);
			}
		}
		p->magic = TASK_MAGIC;
		p->execute = FuncTbl->execute;				/* ֐̃AhX̃Rs[ */
		p->display = FuncTbl->display;
		p->signal = FuncTbl->signal;
		p->flag = _leader->flag;
		p->leader = leader;
		if (_leader->child == NULL) {				/* |C^̂ς */
			_leader->child = p;
		} else {
			for (s = _leader->child ; s->sibling != NULL ; s = s->sibling);
			s->sibling = p;
		}
		return p->body;
    } else {
		return NULL;
    }
}


/*============================================================
 Name: HTTask_CreateChildP
 Desc: ^XÑZbg -- pp[^t
 Kind: Global
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/04/27 (Monday)
 Misc:
 ------------------------------------------------------------
 in  :
 out :
 ============================================================*/
void *HTTask_CreateChildP(const void* leader, int size, const HTTaskFunc *FuncTbl, void *param)
{	
	void *result;

	init_param = param;
	result = HTTask_CreateChild(leader, size, FuncTbl);
	init_param = NULL;
	return result;
}


/*============================================================
 Name: HTTask_CreateRoot
 Desc: [g^XÑZbg
 Kind: Global
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/04/27 (Monday)
 Misc:
 ------------------------------------------------------------
 in  :
 out :
 ============================================================*/
void *HTTask_CreateRoot(int size, const HTTaskFunc * FuncTbl)
{
    register struct taskheader *p;

    if ((p = HTMem_Alloc(size + TASK_HEADER)) != NULL) {
		memset(p, 0, size + TASK_HEADER);
		if (FuncTbl->init != NULL) {
			register FuncP func = (FuncP) FuncTbl->init;
			func(p->body);
		}
		p->magic = TASK_MAGIC;
		p->execute = FuncTbl->execute;
		p->display = FuncTbl->display;
		p->signal = FuncTbl->signal;
		p->flag = M_ACTIVE;
		RootTask = p;
		return p->body;
    } else {
		return NULL;
    }
}


/*============================================================
 Name: HTTask_Sigal
 Desc: VOȋM
 Kind: Global
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/04/27 (Monday)
 Misc:
 ------------------------------------------------------------
 in  :
 out :
 ============================================================*/
void HTTask_Sigal(const void* task, int signal)
{
    register struct taskheader *this = (struct taskheader *) ((char *) task - TASK_HEADER);
    register FuncS func = this->signal;

	if (func != NULL) {
		func(task, signal);
	}
}


/*============================================================
 Name: taskloop
 Desc: w肳ꂽ^XNȉs鉺֐
 Kind: Local
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/04/27 (Monday)
 Misc:
 ------------------------------------------------------------
 in  :
 out :
 ============================================================*/
static void taskloop(struct taskheader* p)
{
    Curtask = p;

    /* ^XNǗ̈悪󂳂ĂȂ`FbN */
#ifdef DEBUG
#if DEBUG == 1
    {
		if (p->magic != TASK_MAGIC) {
			struct memblock* mb;
			struct memblock* mc;
			struct memblock* mp;

			mc = (struct memblock *)((char *)p - HEADER_SIZE);
			/* j󂳂Ă璼Õ^XN̎sAhX\ďI */
			mb = block_chain.next;
			do {
				if (mb == mc) {
					struct taskheader* t;
					t = (struct taskheader *)((char *)mp + TASK_HEADER);
					fprintf(stderr, "Corrupt in function %p\n", t->execute);
					exit(-1);
				}
				mp = mb;
				mb = mb->next;
			} while (mb != &block_chain);
			fprintf(stderr, "Corrupt, but can't find error function.\n");
		}
    }
#endif
#endif

    if (p->flag & M_ACTIVE) {
		register FuncP func;
		switch (task_mode) {
		case TSKMODE_CTRL:
			task_no++;
			func = p->execute;
			goto action;
		case TSKMODE_DRAW:
			func = p->display;
		action:
			if (func != NULL) {
				func(p->body);
			}
			break;
		default:
			break;
		}

		if (p->child != NULL) {
			if (p->child->flag == 0) {				/* qǂIĂ烁 */
				register struct taskheader *child;
				child = p->child;
				p->child = p->child->sibling;
				HTTask_Leave(child);
			}
			if (p->child != NULL) {
				taskloop(p->child);
				/* Z^XN̎s */
				p = p->child;
				while (p->sibling != NULL) {
					if (p->sibling->flag == 0) {	// q̌Z^XNIĂ
						register struct taskheader *sibling;
						sibling = p->sibling;
						p->sibling = p->sibling->sibling;
						HTTask_Leave(sibling);
					} else {
						p = p->sibling;
						taskloop(p);
					}
				}
			}
		}
    }
}


/*============================================================
 Name: HTTask_CtrlLoop
 Desc: ^XN[v
 Kind: Global
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/04/27 (Monday)
 Misc:
 ------------------------------------------------------------
 in  :
 out :
 ============================================================*/
void HTTask_CtrlLoop(void)
{
    register struct taskheader *p;

    p = RootTask;
    task_no = 0;
	task_mode = TSKMODE_CTRL;
    taskloop(p);
}



/*============================================================
 Name: HTTask_DrawLoop
 Desc: `惋[v
 Kind: Global
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/04/27 (Monday)
 Misc:
 ------------------------------------------------------------
 in  :
 out :
 ============================================================*/
void HTTask_DrawLoop(void)
{
    register struct taskheader *p;

    p = RootTask;
	task_mode = TSKMODE_DRAW;
    taskloop(p);
}



/*============================================================
 Name: deleteloop
 Desc: ^XN()
 Kind: Local
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/04/27 (Monday)
 Misc:
 ------------------------------------------------------------
 in  :
 out :
 ============================================================*/
static void deleteloop(struct taskheader* p)
{
    p->flag = 0;									/* ItOZbg(ȉ̊KwSĂ) */
    p->execute = NULL;
    p->display = NULL;
    if (p->signal != NULL) {
		register FuncS func;
		func = p->signal;
		func(p->body, HTTASK_SIG_DELETE);
    }

    if (p->child != NULL) {
		deleteloop(p->child);
		/* Z^XN̎s */
		p = p->child;
		while (p->sibling != NULL) {
			p = p->sibling;
			deleteloop(p);
		}
    }
}


/*============================================================
 Name: HTTask_Delete
 Desc: ^XN̍폜()
 Kind: Global
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/04/27 (Monday)
 Misc:
 ------------------------------------------------------------
 in  :
 out :
 ============================================================*/
void HTTask_Delete(void)
{
    register struct taskheader* p = Curtask;

    deleteloop(p);
}


/*============================================================
 Name: HTTask_DeleteOther
 Desc:
 Kind: Global
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/04/27 (Monday)
 Misc:
 ------------------------------------------------------------
 in  :
 out :
 ============================================================*/
void HTTask_DeleteOther(void* BasePtr)
{
    register struct taskheader* p = (struct taskheader*)((char *)BasePtr - sizeof(struct taskheader));

    deleteloop(p);
}


/*============================================================
 Name: leavetask
 Desc: q[vփ̕Ԋ() 
 Kind: Local
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/04/27 (Monday)
 Misc:
 ------------------------------------------------------------
 in  :
 out :
 ============================================================*/
static void leavetask(struct taskheader* p)
{
    struct taskheader* s;

    /* qǂ̎s */
    if (p->child != NULL) {
		/* qǂ̌Z^XN̎s */
		s = p->child;
		while (s->sibling != NULL) {
			s = s->sibling;
			leavetask(s);
		}
		leavetask(p->child);
		p->child = NULL;
    }
    HTMem_Free(p);
}


/*============================================================
 Name: LeaveTask
 Desc: q[vփ̕Ԋ
 Kind: Global
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/04/27 (Monday)
 Misc:
 ------------------------------------------------------------
 in  :
 out :
 ============================================================*/
static void HTTask_Leave(struct taskheader* p)
{
    struct taskheader* s;

    /* qǂ̎s */
    if (p->child != NULL) {
		/* qǂ̌Z^XN̎s */
		s = p->child;
		while (s->sibling != NULL) {
			s = s->sibling;
			leavetask(s);
		}
		leavetask(p->child);
    }
    HTMem_Free(p);
}


/*============================================================
 Name: HTTask_isActive
 Desc: ^XN݂邩ׂ
 Kind: Global
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/04/27 (Monday)
 Misc:
 ------------------------------------------------------------
 in  :
 out :
 ============================================================*/
int HTTask_isActive(void* task)
{
    register struct taskheader* p = (struct taskheader*)((char *)task - sizeof(struct taskheader));

    if (p->magic = TASK_MAGIC) {
		return 1;
    } else {
		return 0;
    }
}
