/*============================================================
 * Name  : mem_mgr.c
 *
 * Desc  : }l[WNX
 *
 * prefix: HTMem_
 *
 * $Id: htmem_mgr.c,v 1.1 1998/06/24 01:39:41 horita Exp horita $
 *
 * $Log: htmem_mgr.c,v $
 * Revision 1.1  1998/06/24 01:39:41  horita
 * Initial revision
 *
 *============================================================*/

#if DOCUMENT
#endif

/*============================================================
  Include
  ============================================================*/
#include <stdio.h>
#include "htmem_mgr.h"

/*============================================================
  Define
  ============================================================*/
#define	DELTA	64
#define MEM_MAGIC	0xa5
#define LONGSIZE	(sizeof(long))
#define LONG_ALIGN(n)	(((n) + LONGSIZE - 1) & -LONGSIZE)

/*============================================================
  Macro
  ============================================================*/

/*============================================================
  Structure
  ============================================================*/
/*  Rg[ */
struct memblock {
    short magic;									/* }WbNio[B
													   K̒lłȂ
													   ̃ubN͕s
													   ͔j󂳂Ă */
    short occupied;									/*  gpǂ̃tO */
    int size;										/*  ̃ubÑwb_TCY */
    struct memblock *next;							/*  ̃ubNւ̃|C^ */
    struct memblock *prev;							/*  ÕubNւ̃|C^ */
    char body[0];									/*  ubN */
};
#define HEADER_SIZE	(sizeof(struct memblock))

/*============================================================
  Globals
  ============================================================*/

/*============================================================
  Static
  ============================================================*/
static struct memblock block_chain;
static struct memblock *curptr;

/*============================================================
  Local Prototypes
  ============================================================*/



/*============================================================
 Name: HTMem_Init
 Desc: ̏
 Kind: Global
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/04/27 (Monday)
 Misc:
 ------------------------------------------------------------
 in  :
 out :
 ============================================================*/
void HTMem_Init(const void* memory_heap, int size)
{
    struct memblock *h;

    h = (struct memblock *) memory_heap;
    block_chain.size = sizeof(block_chain);
    block_chain.magic = MEM_MAGIC;
    block_chain.occupied = 1;
    block_chain.next = block_chain.prev = h;

    h->size = size;
    h->magic = MEM_MAGIC;
    h->occupied = 0;
    h->next = h->prev = &block_chain;
    curptr = block_chain.next;
}


/*============================================================
 Name: HTMem_Alloc
 Desc: 傫 size ̃ubN蓖Ă
 Kind: Global
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/04/27 (Monday)
 Misc: mۂɎsꍇNULLԂ
 ------------------------------------------------------------
 in  : size.. ubN̑傫
 out : void* mۂubNւ̃|C^
 ============================================================*/
void *HTMem_Alloc(int size)
{
    struct memblock *p;
    struct memblock *s;
    int nbytes;

    nbytes = LONG_ALIGN(size + HEADER_SIZE);
    for (p = block_chain.next ; p != &block_chain; p = p->next) {
		if (!p->occupied && p->size >= nbytes) {
			if (p->size - nbytes > DELTA) {
				s = (struct memblock *) ((char *) p + nbytes);
				s->size = p->size - nbytes;
				s->magic = MEM_MAGIC;
				s->occupied = 0;

				p->next->prev = s;
				s->next = p->next;
				p->next = s;
				s->prev = p;

				p->size = nbytes;
			}
			p->occupied = 1;
			return p->body;
		}
    }
    return NULL;
}


/*============================================================
 Name: HTMem_Free
 Desc: ̊J
 Kind: Global
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/04/27 (Monday)
 Misc: 
 ------------------------------------------------------------
 in  : mۂւ̃|C^
 out : Ȃ
 ============================================================*/
void HTMem_Free(void *block)
{
    struct memblock *mem;

    mem = (struct memblock *) ((char *) block - HEADER_SIZE);

    if (mem->magic != MEM_MAGIC) {
		return;
    }
    if (!mem->prev->occupied) {
		mem->next->prev = mem->prev;
		mem->prev->next = mem->next;
		mem->prev->size += mem->size;
		mem = mem->prev;
    }
    if (!mem->next->occupied) {
		mem->next->next->prev = mem;
		mem->size += mem->next->size;
		mem->next = mem->next->next;
    }
    mem->occupied = 0;
}

