/*============================================================
 * Name  : main.c
 *
 * Desc  : Ninja Main
 *
 * $Id$
 *
 * $Log$
 *============================================================*/

/*============================================================
  Include
  ============================================================*/
#include <stdio.h>
#include <ninjawin.h>
#include <njdef.h>
#include "root.h"
#include "httask_mgr.h"

/*============================================================
  Define
  ============================================================*/
#define MAX_MATRIX		32
#define MAX_VERTEX		60000
#define MAX_VERTEX_BUFF	10000

/*============================================================
  Macro
  ============================================================*/

/*============================================================
  Structure
  ============================================================*/

/*============================================================
  Globals
  ============================================================*/
NJS_VIEW 		view;
NJS_MATRIX 		matrix[MAX_MATRIX];
NJS_VERTEX_BUF	vbuf[MAX_VERTEX_BUFF];
NJS_LIGHT		light;

/*============================================================
  Static
  ============================================================*/
static int frame;
static int wframe;
static int time;

/*============================================================
  Local Prototypes
  ============================================================*/


/*============================================================
 Name: njUserInit
 Desc: [U[֐
 Kind: Global
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/03/23 (Monday)
 Misc:
 ------------------------------------------------------------
 in  :	
 out :
 ============================================================*/
void njUserInit(void)
{
	extern void Init_Task(void);

    njInitSystem(NJD_RESOLUTION_VGA, NJD_FRAMEBUFFER_MODE_ARGB1555, 1);
    njClipZ(0.0, -30000.0);
    njInitVertexBuffer( MAX_VERTEX, 0, MAX_VERTEX, 0);
    njInit3D( vbuf, MAX_VERTEX_BUFF );
    njInitMatrix(matrix, MAX_MATRIX, 0);
    njInitView(&view);
	njSetView(&view);
    njSetPerspective(NJM_DEG_ANG(45.0));
    njControl3D(NJD_CONTROL_3D_CONSTANT_ATTR);
    njSetConstantAttr(0xffffffff, NJD_FILTER_BILINEAR|NJD_FLAG_USE_ALPHA);
    njSetBackColor(0xff000000, 0xff000000, 0xff000000);

    njCreateLight( &light, NJD_AMBIENT );
    njSetLightIntensity(&light, 1.0, 1.0, 1.0);

	/* counter */
	wframe = 0;
	time = GetCurrentTime();
	njPrintSize(16);

	/* Task */
	set_root_task();
}


/*============================================================
 Name: njUserMain
 Desc: [U[C֐
 Kind: Global
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/03/23 (Monday)
 Misc:
 ------------------------------------------------------------
 in  :	
 out :
 ============================================================*/
Sint32 njUserMain(void)
{
	HTTask_CtrlLoop();
	HTTask_DrawLoop();

	/* Frame rate */
	wframe++;
	if (GetCurrentTime() > (time + (5 * 1000))) {
		frame = wframe / 5;
		wframe = 0;
		time = GetCurrentTime();
	}
	njPrintD(NJM_LOCATION(2, 4), frame, 3);
	njPrintC(NJM_LOCATION(6, 4), "frame/s");

	return NJD_USER_CONTINUE;
}


/*============================================================
 Name: njUserExit
 Desc: [U[IԐ
 Kind: Global
 User: Eiji Horita horita@cs1.sega.co.jp
 Date: 1998/03/23 (Monday)
 Misc:
 ------------------------------------------------------------
 in  :	
 out :
 ============================================================*/
void njUserExit(void)
{
	njExitSystem();
}
