#
# Common Makefile 
#
# Written by Horita <horita@cs1.sega.co.jp>
# Time-stamp: <horita: 1997N1216>

#
# Ninja Library Path
#
NJDIR	= d:/usr/local/ninja
NJLIB	= $(NJDIR)/lib
NJINC	= $(NJDIR)/include

#
# Program
#
CC	= cl.exe /nologo /W3
LD	= link.exe /nologo -debug -debugtype:both
RM	= rm -f
MKDEP	= mkdep
AR	= lib /nologo

PDBFILE	= $(subst .lib,.pdb,$(subst .exe,.pdb,$(TARGET)))
MAPFILE	= $(subst .lib,.map,$(subst .exe,.map,$(TARGET)))
#
# option
#
CCFLAGS	= -J -I$(NJINC) $(CFLAGS) -O2 -G6 -od -Zi -Fd$(PDBFILE) 
ifdef DEBUG
CCFLAGS += -DDEBUG=$(DEBUG)
endif

#
# define
#
DEPEND	= depend.mak

#
# Macro
#
OBJDIR	= obj
OBJS	= $(addprefix $(OBJDIR)/, $(subst .c,.obj,$(subst .cpp,.obj,$(SRCS))))
NJLIBS	= $(NJLIB)/Nindows.lib $(NJLIB)/NjWin.lib $(NJLIB)/Ninja.lib $(NJLIB)/KAMUI.LIB
SYSLIB	= $(NJLIBS) user32.lib kernel32.lib 

ifndef TARGET
TARGET	= test.exe
endif

ifndef MAKEFILE
MAKEFILE = Makefile
endif

#
# Default Rule
#
.SUFFIXES: .c .cpp .obj .exe .lib

$(OBJDIR)/%.obj : %.c
	$(CC) $(CCFLAGS)  -c $< -Fo$@

$(OBJDIR)/%.obj : %.cpp
	$(CC) $(CCFLAGS)  -c $< -Fo$@

%.asm : %.cpp
	$(CC) $(CCFLAGS)  -c $< -Fa$@ -Fo/tmp/foo

%.asm : %.c
	$(CC) $(CCFLAGS)  -c $< -Fa$@ -Fo/tmp/foo

#
# Build TARGET from suffix
#
all : $(TARGET)

ifeq ($(suffix $(TARGET)), .exe)
$(TARGET) : $(OBJS) $(LIBS) $(MAKEFILE)
	$(LD) /subsystem:windows $(LDFLAGS) -map:$(MAPFILE) -pdb:$(PDBFILE) -OUT:$@ $(OBJS) $(LIBS) $(SYSLIB)
endif

ifeq ($(suffix $(TARGET)), .lib)
$(TARGET): $(OBJS)
	$(AR) /OUT:$@ /SUBSYSTEM:WINDOWS $(OBJS) 
endif

#
# Maintenance Target
#
default: all

.PHONY: clean rebuild depend

clean:
	$(RM) $(OBJS) $(TARGET) $(TARGET:.exe=.pdb)

rebuild:
	$(MAKE) clean all

depend:
	@test -f $(DEPEND) || touch $(DEPEND)
	$(MKDEP) -f $(DEPEND) $(CCFLAGS) $(SRCS)
	@cp $(DEPEND) $(DEPEND).old
	@sed "s|\(.*\.obj\)|$(OBJDIR)/\1|g" $(DEPEND).old > $(DEPEND)
	@rm $(DEPEND).old

-include $(DEPEND)

$(DEPEND) : $(MAKEFILE)
	@test -f $(DEPEND) || touch $(DEPEND)
	$(MKDEP) -f $(DEPEND) $(CCFLAGS) $(SRCS)
	@cp $(DEPEND) $(DEPEND).old
	@sed "s|\(.*\.obj\)|$(OBJDIR)/\1|g" $(DEPEND).old > $(DEPEND)
	@rm $(DEPEND).old

